/*
 * Hamsam - Instant Messaging API
 * Copyright (C) 2003 Raghu K
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package gui.action;

import java.util.Date;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Vector;

import javax.swing.JDialog;
import javax.swing.JInternalFrame;
import javax.swing.JOptionPane;

import hamsam.api.Response;
import hamsam.api.IMListener;
import hamsam.api.Buddy;
import hamsam.api.Message;
import hamsam.api.Conference;
import hamsam.api.MessageComponent;
import hamsam.api.TextComponent;
import hamsam.api.SmileyComponent;
import hamsam.api.URLComponent;
import hamsam.exception.HamsamException;
import hamsam.exception.IllegalStateException;
import hamsam.exception.UnsupportedOperationException;

import hamsam.protocol.Protocol;

import gui.ChatPanel;
import gui.MainFrame;

import gui.tools.PropertiesManager;
import gui.tools.Traces;

public class IMListenerForGui
    implements IMListener {
	static HashMap chatPanels = new HashMap();
	static HashMap buddyChatPanels = new HashMap();
	
	public static void registerChatPanel(Conference c, ChatPanel cp) {
		chatPanels.put(c, cp);
	}
	public static void unregisterChatPanel(Conference c) { 
		chatPanels.remove(c);
	}
	public static void registerChatPanel(String username, ChatPanel cp) { 
		buddyChatPanels.put(username, cp);
	}
	public static void unregisterChatPanel(String username) {
		buddyChatPanels.remove(username);
	}
	
  private PropertiesManager propertiesMgr = null;
  private Buddy[] buddiesReceived = null;
  private Buddy[] buddiesIgnored = null;
  private boolean isConnected = false;
  
  public void setPropertiesMgr(PropertiesManager p) {
    this.propertiesMgr = p;
  }

  public Buddy[] getBuddyListReceived() {
    return this.buddiesReceived;
  }

  public Buddy[] getBuddyListIgnored() {
    return this.buddiesIgnored;
  }

  public void connecting(Protocol protocol) {
    Traces.printTraces("Connecting...");
    MainFrame.myRef.setStatus(propertiesMgr.getValue("connecting"));
  }

  public void connected(Protocol protocol) {
    Traces.printTraces("Connected");
    this.isConnected = true;
    MainFrame.myRef.enableContactPanel(true);
    MainFrame.myRef.setStatus(propertiesMgr.getValue("connected"));
    MainFrame.myRef.updateBuddyListReceived(this.buddiesReceived);
    MainFrame.myRef.setConnected(true);
    
  }

  public void connectFailed(Protocol protocol, String message) {
    Traces.printTraces("Connect failed: " + message);
    MainFrame.myRef.setStatus(propertiesMgr.getValue("connectFailed") + ": "  + message);
    MainFrame.myRef.setConnected(false);
  }

  public void disconnected(Protocol protocol) {
    Traces.printTraces("Disconnected: "/* + Thread.currentThread().activeCount()*/);
    MainFrame.myRef.enableContactPanel(false);
    MainFrame.myRef.setStatus(propertiesMgr.getValue("deconnected"));
    MainFrame.myRef.setConnected(false);
  }

  public void buddyListReceived(Protocol protocol, Buddy[] buddies) {
    buddiesReceived = buddies;
    Traces.printTraces("Buddy list received: ");
    for (int i = 0; i < buddies.length; i++)
      System.out.println(buddies[i].getUsername());
  }

  public void ignoreListReceived(Protocol protocol, Buddy[] buddies) {
    buddiesIgnored = buddies;
    Traces.printTraces("Ignore list received: ");
    for (int i = 0; i < buddies.length; i++)
      System.out.println(buddies[i].getUsername());
  }

  public void protocolMessageReceived(Protocol protocol, Message msg) {
    printMessage("Protocol Message: ", msg);
  }

  public void instantMessageReceived(Buddy buddy, Message msg) {
    printMessage(buddy.getUsername() + ": ", msg);
    ChatPanel panel = (ChatPanel)buddyChatPanels.get(buddy.getUsername());
    
    if(panel == null) {
    	Vector v = new Vector();
    	v.add(buddy);
    	panel = MainFrame.myRef.addChatPanel(v);
    }
    panel.messageRecieved(buddy, msg);
  }

  public void offlineMessageReceived(Buddy buddy, Date time, Message msg) {
    printMessage(buddy.getUsername() + " [offline @ " + time + "] : ", msg);
  }

  public void mailNotificationReceived(Protocol protocol, int count,
                                       String[] from, String[] subject) {
    if (count == -1)
      Traces.printTraces("You have new mail");
    else
      Traces.printTraces("You have " + count + " new mail(s).");

    if (from != null && subject != null) {
      for (int i = 0; i < from.length || i < subject.length; i++)
        Traces.printTraces("\t(" + (i + 1) + ") from: " + from[i] +
                           " subject: " + subject[i]);
    }
  }

  public void conferenceInvitationAccepted(Conference conf, Buddy buddy) {
    System.out.println(buddy.getUsername() + " accepted the invitation to conference: " + conf);
    JOptionPane.showMessageDialog(MainFrame.myRef, buddy.getUsername() + " has joined the conference.");
  }

  public void conferenceInvitationDeclined(Conference conf, Buddy buddy,
                                           String message) {
    System.out.println(buddy.getUsername() +
                       " declined to join the conference: " + conf +
                       " with the message: " + message);
    JOptionPane.showMessageDialog(MainFrame.myRef, buddy.getUsername() + " has decided not to join the conference: " + message);
  }

  public Response conferenceInvitationReceived(Conference conf, String message) {
    Buddy[] parts = conf.getParticipants();
    System.out.print("Conference invitation: " + message);

    for (int i = 0; i < parts.length; i++)
      System.out.print(parts[i].getUsername() + " ");

    System.out.println();

    StringBuffer sb = new StringBuffer();
    for(int i = 0; i < parts.length; i++) {
    	sb.append(" " + parts[i].getUsername() );
    }
    boolean accept = (0 == JOptionPane.showOptionDialog(MainFrame.myRef, "Do you want to join a conference with" + sb.toString() + "?\nThe message is:\n" + message, 
    							 "Join conference?", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE, 
    							 null, null, null));
    if(accept) { 
    	ChatPanel cp = MainFrame.myRef.addChatPanel(new Vector());
    	cp.setConference(conf);
    	chatPanels.put(conf, cp);
    	return new Response();
    } else { 
    	return new Response("Conference inviatation declined.");
    }
  }

  public void conferenceMessageReceived(Conference conf, Buddy buddy,
                                        Message message) {
    System.out.print(conf);
    printMessage(" conference: " + buddy.getUsername() + ": ", message);
    ChatPanel cp = (ChatPanel)chatPanels.get(conf);
    cp.messageRecieved(buddy, message );
  }

  public void buddyStatusChanged(Buddy buddy) {
    Traces.printTraces("Status of " + buddy.getUsername() + " is now " +
                       buddy.getStatus());
  }

  public void typingStarted(Buddy buddy) {
    System.out.println(buddy.getUsername() + " is typing");
  }

  public void typingStopped(Buddy buddy) {
    System.out.println(buddy.getUsername() + " stopped typing");
  }

  public void buddyAdded(Buddy buddy) {
	Buddy[] newBuddyList = new Buddy[buddiesReceived.length+1];
	for(int i = 0; i < buddiesReceived.length; i++) {
		newBuddyList[i] = buddiesReceived[i];
	}
	newBuddyList[newBuddyList.length -1] = buddy;
	buddiesReceived = newBuddyList;
	MainFrame.myRef.tabbedPanel.contactMgrPanel.addBuddySucceeded(buddy);
    System.out.println(buddy.getUsername() + " is added to the buddy list");
  }

  public void buddyAddRejected(Buddy buddy, String reasonMessage) {
    System.out.println(buddy.getUsername() + " rejected to be a buddy: " +
                       reasonMessage);
    JOptionPane.showMessageDialog(MainFrame.myRef, 
			  "Request to add buddy " + buddy.getUsername() + " rejected.  Reason: " + reasonMessage, "Buddy not added", 
			  JOptionPane.ERROR_MESSAGE);
  }

  public void buddyAddFailed(Buddy buddy, String reasonMessage) {
    Traces.printTraces("Attempt to add " + buddy.getUsername() + " failed: " +
                       reasonMessage);
    JOptionPane.showMessageDialog(MainFrame.myRef, 
			  "Error adding " + buddy.getUsername() + " to your buddy list.  Reason: " + reasonMessage, "Buddy not added", 
			  JOptionPane.ERROR_MESSAGE);
  }

  public Response buddyAddRequest(Buddy buddy, Buddy myself, String message) {
    System.out.println(buddy.getUsername() + " is trying to add " +
                       myself.getUsername() + " to his buddy list: " + message);
    
    int choice = JOptionPane.showOptionDialog(MainFrame.myRef, 
    							"Do you want to be on " + buddy.getUsername() + "'s buddy list?  Message is \"" + message + "\"", 
    							"Buddy add request for " + myself.getUsername(), 
    							JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE, null, new Object[] { "Yes", "No"} , "Yes");

    return (choice == 0) ? new Response() : new Response("I don't want to be your buddy."); 				

  }

  public void buddyDeleted(Buddy buddy) {
    System.out.println(buddy.getUsername() + " is deleted from the buddy list");
    MainFrame.myRef.tabbedPanel.contactMgrPanel.deleteBuddy(buddy);
    JOptionPane.showMessageDialog(MainFrame.myRef, 
			  "Deleted " + buddy.getUsername() + " from your buddy list.",
			  "Buddy deleted", 
			  JOptionPane.INFORMATION_MESSAGE);
  }

  public void buddyDeleteFailed(Buddy buddy, String reasonMessage) {
    Traces.printTraces("Attempt to delete " + buddy.getUsername() + " failed: " +
                       reasonMessage);
    JOptionPane.showMessageDialog(MainFrame.myRef, 
			  "Error deletng " + buddy.getUsername() + " from your buddy list.  Reason " + reasonMessage, 
			  "Buddy not deleted", 
			  JOptionPane.ERROR_MESSAGE);
  }

  public void buddyIgnored(Buddy buddy) {
    System.out.println(buddy.getUsername() + " is ignored now");
  }

  public void buddyIgnoreFailed(Buddy buddy, String reasonMessage) {
    Traces.printTraces("Attempt to ignore " + buddy.getUsername() + " failed: " +
                       reasonMessage);
  }

  public void buddyUnignored(Buddy buddy) {
    System.out.println(buddy.getUsername() + " is unignored now");
  }

  public void buddyUnignoreFailed(Buddy buddy, String reasonMessage) {
    Traces.printTraces("Attempt to unignore " + buddy.getUsername() +
                       " failed: " + reasonMessage);
  }

  private void printMessage(String header, Message msg) {
    System.out.print(header);

    Enumeration e = msg.getComponents();
    while (e.hasMoreElements()) {
      MessageComponent comp = (MessageComponent) e.nextElement();
      if (comp instanceof TextComponent) {
        TextComponent txt = (TextComponent) comp;
        System.out.print("<text>" + new String(txt.getSequence()));
      }
      else if (comp instanceof SmileyComponent) {
        SmileyComponent sml = (SmileyComponent) comp;
        System.out.print("<smiley>" + sml.getName());
      }
      else if (comp instanceof URLComponent) {
        URLComponent url = (URLComponent) comp;
        System.out.print("<url>" + url.getLinkText());
      }
      else
        System.out.println(comp);
    }

    System.out.println();
  }

public void conferenceParticipantJoined(Conference conf, Buddy buddy)
{
	System.out.println(buddy.getUsername() + " joined the conference: " + conf);
	ChatPanel cp = (ChatPanel)chatPanels.get(conf);
    cp.buddyVector.add(buddy);
    cp.model.addElement(buddy.getUsername());
    conf.addParticipant(buddy);
}

public void conferenceParticipantLeft(Conference conf, Buddy buddy) {
  System.out.println(buddy.getUsername() + " left the conference: " + conf);
  ChatPanel cp = (ChatPanel)chatPanels.get(conf);
  cp.buddyVector.removeElement(buddy);
  cp.model.removeElement(buddy.getUsername());
  conf.removeParticipant(buddy);
}

public void conferenceClosed(Conference conf)
{
	try {
		ChatPanel cp = (ChatPanel) chatPanels.get(conf);
//		MainFrame.myRef.chatPanels.remove(chatPanel.)
//		chatPanels.remove(chatPanel.getPanelName());
		MainFrame.myRef.tabbedPanel.remove(cp);
//	this.tabbedPanel.removeTabAt(this.tabbedPanel.getSelectedIndex());
//	Conference c = chatPanel.getConference();	 
		conf.quit();
		chatPanels.remove(conf);
	} catch (UnsupportedOperationException e) {
		e.printStackTrace();
	} catch (IllegalStateException e) {
		e.printStackTrace();
	}
}
public void sendMessageSucceeded(Message message) {
	// TODO Auto-generated method stub
	
}
public void sendMessageFailed(Message message) {
	// TODO Auto-generated method stub
	
}
}