/**
 * 
 */
package hamsam.protocol.insecureSample;

import hamsam.api.Buddy;
import hamsam.api.Conference;
import hamsam.api.IMListener;
import hamsam.api.Message;
import hamsam.api.MessageComponent;
import hamsam.api.Response;
import hamsam.api.SmileyComponent;
import hamsam.api.TextComponent;
import hamsam.exception.IllegalArgumentException;
import hamsam.exception.IllegalStateException;
import hamsam.exception.UnsupportedOperationException;
import hamsam.net.ProxyInfo;
import hamsam.protocol.Protocol;

import java.rmi.dgc.VMID;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Set;
import java.util.Iterator;
import java.io.Serializable;

/**
 * 
 * There is one InsecureServerLogic instance per client connection.  
 * Static variables are used to store information like conference 
 * attendees and buddy lists between connections.
 * 
 * @author Ivan Tam
 *
 * TODO RCS Does it still make sense to have this implement Protocol?
 */

public class InsecureProtocol implements Protocol, Serializable {
	
	/** This hashmap maps from Strings (GUIDs) to Conference objects. */
	protected Hashtable conferences = new Hashtable();

	/** 
	 *  Ideally, InsecureProtocol would extend Thread, but InsecureProtocol lives as 
	 *  long as the client does, and Thread objects may not be restarted once they
	 *  have stopped.  Therefore, we need this inner class to implement the client 
	 *  Protocol event loop.
	 */
	private class InsecureProtocolThread extends Thread { 
		InsecureProtocol p;
		public InsecureProtocolThread(InsecureProtocol p) { 
			super("<put root password here>");
			this.p = p;
		}
		public void run () {
			while(serverHandle.receiveMessageFromServer(imListener, p)) { System.out.println("Got message."); }
			System.out.println("Connection done...");
			p.thread = null;
			serverHandle.close();
			
		}
	}
	private static final long serialVersionUID = -6157046037053722251L;

	/**
	 *  We want to make the InsecureProtocol constructor 
	 *  private because hamsam makes us barf if we pass it more than one 
	 *  instance.
	 *  
	 *  (Note:  limiting ourselves to one insecure protocol might make 
	 *  load testing hard, so we could add the public constructor back in later...)
	 */
	public static final Protocol instance = new InsecureProtocol();
	
	/** 
	 * The username of the buddy associated with this Protocol instance.
	 */
//	private Buddy buddy = null;
	
	
	/**
	 * A map from a Buddy to a set of buddies.
	 */
	private static Hashtable buddyLists = new Hashtable();

	/**
	 * A convenience function to save us from writing the cast over and over again.
	 */
//	private IMListener getListener(Buddy b) { 		return (IMListener)buddyListeners.get(b);	}

	InsecureNetworking serverHandle;
	
	private InsecureProtocol() {
	}

	public void changeStatus(String status) throws IllegalArgumentException,
			IllegalStateException {
		serverHandle.sendMessageToServer(InsecureNetworking.CHANGE_STATUS, new Serializable[] { status } );
		//if(success.booleanValue()) { 
		//	getListener(this.buddy).buddyStatusChanged(this.buddy);
		//	this.buddy.setStatus(status);
		//} else { 
		//	getListener(this.buddy).buddyStatusChangeFailed(this.buddy);  <- This call doesn't really exist yet. :(
		//}
	}
	private InsecureProtocolThread thread; 
	public void connect(String username, String password, String server, ProxyInfo info)
			throws IllegalStateException {

		serverHandle = new InsecureNetworking(username, password, server, info, this);
		serverHandle.sendMessageToServer(InsecureNetworking.CONNECT, new Serializable[] { username, password });

		if(thread != null) { 
			throw new java.lang.UnsupportedOperationException
			  ("Connect called even though this InsecureProtocol instance is still connected to the server!");
		}
		
		thread = new InsecureProtocolThread(this);
		thread.start();
	}

	public void disconnect() throws IllegalStateException {
		serverHandle.sendMessageToServer(InsecureNetworking.DISCONNECT, new Serializable[] {});
		// The server will send us a "DISCONNECTED" message.  Then we clean up the conection state, etc...
	}

	public String getProtocolName() {
		return "Insecure CS 161 sample protocol";
	}

	public boolean isBuddyAddRequestSupported() { 	return true;	}
	public boolean isBuddyNameAliasSupported() {	return false;	}
	public boolean isBuddyGroupSupported() {		return false;	}
	public boolean isIgnoreSupported() {			return true;	}
	public boolean isOfflineMessageSupported() {	return false;	}
	public boolean isTypingNotifySupported() {		return false;	}
	public boolean isConferenceSupported() {		return true;	}
	public boolean isMailNotifySupported() {		return false;	}
	public boolean isInvisibleSupported() {			return false;	}

	private IMListener imListener = null;

   public void setListener(IMListener listener) {
       this.imListener = listener;
	}

	public void startConference(Conference conf, String message)
			throws UnsupportedOperationException, IllegalStateException {
		String[] invitees = new String[conf.getParticipants().length];
		for(int i = 0; i < invitees.length; i++) { 
			invitees[i] = conf.getParticipants()[i].getUsername();
		}
		String guid = new VMID().toString(); // Why would the GUID implementation be named this?!?  Note that Java 1.5's UUID's are just based on time, not network address. 
		conferences.put(guid, conf);
		serverHandle.sendMessageToServer(InsecureNetworking.START_CONFERENCE, new Serializable[] { guid, message, invitees });
	}

	public void quitConference(Conference conf)
			throws UnsupportedOperationException, IllegalStateException {
		if (conferences.contains(conf)) {
			Set keys = conferences.keySet();
			Iterator it = keys.iterator();
			String guid = null;
			while(it.hasNext()) {
				guid = (String)it.next();
				if (conferences.get(guid).equals(conf))
					break;
			}
			Conference c = (Conference)conferences.remove(guid);
			serverHandle.sendMessageToServer(InsecureNetworking.QUIT_CONFERENCE, new Serializable[] { guid } );
			conferences.remove(conf);
		} else {
			System.out.println("quitConference: Error conf not found");
		}
	}

	public void sendConferenceMessage(Conference conf, Message message)
			throws UnsupportedOperationException, IllegalStateException {
		if (conferences.contains(conf)) {
			Set keys = conferences.keySet();
			Iterator it = keys.iterator();
			String guid = null;
			while(it.hasNext()) {
				guid = (String)it.next();
				if (conferences.get(guid).equals(conf))
					break;
			}
			serverHandle.sendMessageToServer(InsecureNetworking.SEND_CONFERENCE_MESSAGE, new Serializable[] {guid, message});
		} else {
			System.out.println("sendConferenceMessage: Error conf not found");
		}			
	}

	public void addToBuddyList(Buddy buddy) throws IllegalArgumentException,
			IllegalStateException {
		// TODO RCS The server needs an add request message, but addToBuddyList doesn't provide one, so I hardcoded something here...
		serverHandle.sendMessageToServer(InsecureNetworking.ADD_TO_BUDDY_LIST, new Serializable[] { buddy.getUsername(), "Will you be my buddy" });
	}

	public void deleteFromBuddyList(Buddy buddy) {
		System.out.println("Sending request to delete buddy " + buddy.getUsername() + " XXX");
		serverHandle.sendMessageToServer(InsecureNetworking.DELETE_FROM_BUDDY_LIST, new Serializable[] { buddy.getUsername() });
	}

	public void sendInstantMessage(Buddy buddy, Message message)
			throws IllegalStateException {
		serverHandle.sendMessageToServer(InsecureNetworking.SEND_INSTANT_MESSAGE, new Serializable[] { buddy.getUsername(), message });
	}

	public SmileyComponent[] getSupportedSmileys()  {
		// TODO RCS Add smileys?
		return new SmileyComponent[] { };
	}

	public String[] getSupportedStatusMessages() {
		return new String[] { "Online", "Offline" };
	}

	// --- Our protocol doesn't support any of the calls defined below this line.
	
	public void ignoreBuddy(Buddy buddy) throws UnsupportedOperationException,
			IllegalStateException {
		throw new UnsupportedOperationException("Ignore buddy notifications are not supported by this protocol");
	}

	public void unignoreBuddy(Buddy buddy)
			throws UnsupportedOperationException, IllegalStateException {
		throw new UnsupportedOperationException("Ignore buddy notifications are not supported by this protocol");
	}

	public void typingStarted(Buddy buddy)
			throws UnsupportedOperationException, IllegalStateException {
		throw new java.lang.UnsupportedOperationException("Typing started notifications are not supported by this protocol");
	}

	public void typingStopped(Buddy buddy)
			throws UnsupportedOperationException, IllegalStateException {
		throw new java.lang.UnsupportedOperationException("Typing stopped notifications are not supported by this protocol");
	}

	public void changeBuddyAlias(Buddy buddy, String alias)
			throws UnsupportedOperationException {
		throw new java.lang.UnsupportedOperationException("Buddy aliases are not supported by this protocol");
	}

}
